" Plugin to update the %changelog section of RPM spec files
" This is a combination of ideas and code from
" spec.vim by Gustavo Niemeyer <niemeyer@conectiva.com>
" spec_ftplugin.vim by Guillaume Rousse <rousse@ccr.jussieu.fr>
" and Sir Raorn <raorn@altlinux.ru>
" add_changelog utility by Dmitry V. Levin <ldv@altlinux.ru>
" Filename: spec_alt.vim
" Version: 0.3

if exists("b:did_altchangelog_ftplugin") | finish | endif

let b:did_altchangelog_ftplugin = 1

" Avoid problems if running in 'compatible' mode.
let s:keepcpo = &cpo
set cpo&vim

if !exists("no_plugin_maps") && !exists("no_spec_maps")
    if !hasmapto("<Plug>SpecAltChangelog")
	map <buffer> <LocalLeader>ac <Plug>SpecAltChangelog
    endif
endif

noremap <buffer> <unique> <script> <Plug>SpecAltChangelog :call <SID>SpecAltChangelog()<CR>

if !exists("g:spec_chglog_get_specfile_evr_method")
    let g:spec_chglog_get_specfile_evr_method = "gear"
endif

" Useful settings
setlocal makeprg=rpm\ -ba\ %
setlocal errorformat=error:\ line\ %l:\ %m

" Helper functions
function! s:Date()
    let savelang = v:lc_time
    execute "language time C"
    let date = strftime("%a %b %d %Y")
    execute "language time " . savelang
    return date
endfunction

" Return location of first line matching
" the given pattern after the given line
" Return -1 if not found at the end of the file
function! s:GetFirstLocation(from, pattern)
    let linenb = a:from
    while (linenb <= line("$"))
	let linenb = linenb + 1
	let linestr = getline(linenb)
	if (linestr =~ a:pattern)
	    return linenb
	endif
    endwhile
    return -1
endfunction

function! s:GetSystemLine(command)
    let out = systemlist(a:command)
    if v:shell_error != 0
	return ""
    endif
    return out[0]
endfunction

function! s:GetPackager()
    if exists("g:spec_chglog_packager")
	return g:spec_chglog_packager
    endif

    let packager = s:GetSystemLine("rpm --eval '%{?packager}'")
    if packager == ""
	let packager = s:GetSystemLine("eval \"$(hsh --printenv)\" 2>/dev/null; echo ${packager:-}")
    endif
    if packager == ""
	let name = s:GetSystemLine("git config --get user.name")
	let email = s:GetSystemLine("git config --get user.email")
	if name != "" && email != ""
	    let packager = name . " <" . email . ">"
	endif
    endif
    if packager == ""
	redraw | echohl Error | echomsg "error: undefined packager (configure in ~/.rpmmacros, ~/.hasher/config, or ~/.gitconfig)." | echohl None
	return
    endif
    return packager
endfunction

function! s:GetSpecFileEVR(spec)
    let evr = ""
    if g:spec_chglog_get_specfile_evr_method == "gear"
	let evr = s:GetSystemLine("set -- $(describe-specfile --epoch ".shellescape(a:spec)."); echo \"$2-$3\"")
    elseif g:spec_chglog_get_specfile_evr_method == "rpm"
	let evr = s:GetSystemLine("rpm -q --qf '%|EPOCH?{%{EPOCH}:}|%{VERSION}-%{RELEASE}\n' --specfile ".shellescape(a:spec)." 2>/dev/null|head -1")
    endif
    if evr == ""
	redraw | echohl Error | echomsg "Failed to parse EVR from specfile" | echohl None
    endif
    return evr
endfunction

" Main function
function! s:SpecAltChangelog()
    let line = 0
    let packager = s:GetPackager()
    if (packager == "")
	return
    endif
    let date = s:Date()
    let tmpfile = tempname()
    silent execute 'w ' . tmpfile
    let evr = s:GetSpecFileEVR(tmpfile)
    if (evr == "")
	return
    endif
    let full = "* ".date." ".packager." ".evr
    let chgline = s:GetFirstLocation(0, '^%changelog')
    if (chgline == -1)
	let option = confirm("Can't find %changelog. Create one? ","&End of file\n&Here\n&Cancel",3)
	if (option == 1)
	    call append(line("$"),"")
	    call append(line("$"),"%changelog")
	    execute line("$")
	    let chgline = line(".")
	elseif (option == 2)
	    call append(line("."),"%changelog")
	    normal j
	    let chgline = line(".")
	elseif (option == 3)
	    return
	endif
    endif

    let insert_line = 0
    if (getline(chgline+1) != full)
	let lastline = line("$")
	call append(chgline, full)
	call append(chgline+1,"- ")
	let insert_line = chgline+2
	if (chgline != lastline)
	    call append(chgline+2,"")
	endif
    else
	let line = chgline
	if !exists("g:spec_chglog_prepend")
	    while !(getline(line+2) =~ '^\( *\|\*.*\)$')
		let line = line+1
	    endwhile
	endif
	call append(line+1,"- ")
	let insert_line = line+2
    endif
    execute insert_line
    startinsert!
endfunction

let &cpo = s:keepcpo
unlet s:keepcpo
